package com.widget.imagevideobanner.player;

import android.app.Application;
import android.net.Uri;
import android.util.Log;
import android.view.LayoutInflater;

import com.google.android.exoplayer2.DefaultLoadControl;
import com.google.android.exoplayer2.ExoPlayerFactory;
import com.google.android.exoplayer2.SimpleExoPlayer;
import com.google.android.exoplayer2.source.ExtractorMediaSource;
import com.google.android.exoplayer2.source.MediaSource;
import com.google.android.exoplayer2.trackselection.DefaultTrackSelector;
import com.google.android.exoplayer2.ui.PlayerView;
import com.google.android.exoplayer2.upstream.DefaultHttpDataSourceFactory;
import com.google.android.exoplayer2.upstream.FileDataSourceFactory;
import com.google.android.exoplayer2.upstream.cache.Cache;
import com.google.android.exoplayer2.upstream.cache.CacheDataSinkFactory;
import com.google.android.exoplayer2.upstream.cache.CacheDataSource;
import com.google.android.exoplayer2.upstream.cache.CacheDataSourceFactory;
import com.google.android.exoplayer2.upstream.cache.LeastRecentlyUsedCacheEvictor;
import com.google.android.exoplayer2.upstream.cache.SimpleCache;
import com.google.android.exoplayer2.util.Util;
import com.widget.imagevideobanner.BaseApplication;
import com.widget.imagevideobanner.R;

import java.io.File;
import java.util.HashMap;

public class PlayerManager {

    private  ExtractorMediaSource.Factory networkMediaSourceFactory;
    private  ExtractorMediaSource.Factory localMediaSourceFactory;

    public SimpleExoPlayer exoPlayer;
    public PlayerView playerView;
    public String playUrl;
    private static PlayerManager mInstance;
    private HashMap<String,MediaSource> mediaSourceHashMap = new HashMap<>();

    public PlayerManager() {
        Application application = BaseApplication.getApplication();
        buildLocalMediaSourceFactory(application);
        buildNetworkMediaSourceFactory(application);
        buildPlayerAndView(application);
    }

    private void buildPlayerAndView(Application application){
        //创建exoplayer播放器实例
        exoPlayer = ExoPlayerFactory.newSimpleInstance(application,
                //视频的音视频轨道如何加载,使用默认的轨道选择器
                new DefaultTrackSelector(),
                //视频缓存控制逻辑,使用默认的即可
                new DefaultLoadControl());
        //加载咱们布局层级优化之后的能够展示视频画面的View
        playerView = (PlayerView) LayoutInflater.from(application).inflate(R.layout.layout_exo_player_view, null, false);
        //别忘记 把播放器实例 和 playerView，controlView相关联
        //如此视频画面才能正常显示,播放进度条才能自动更新
        playerView.setPlayer(exoPlayer);
    }

    private void  buildNetworkMediaSourceFactory(Application application){
        DefaultHttpDataSourceFactory dataSourceFactory = new DefaultHttpDataSourceFactory(Util.getUserAgent(application, application.getPackageName()));        //创建缓存，指定缓存位置，和缓存策略,为最近最少使用原则,最大为200m
        Cache cache = new SimpleCache(application.getCacheDir(), new LeastRecentlyUsedCacheEvictor(1024 * 1024 * 200));
        //把缓存对象cache和负责缓存数据读取、写入的工厂类CacheDataSinkFactory 相关联
        CacheDataSinkFactory cacheDataSinkFactory = new CacheDataSinkFactory(cache, Long.MAX_VALUE);
        CacheDataSourceFactory cacheDataSourceFactory = new CacheDataSourceFactory(cache,
                dataSourceFactory,
                new FileDataSourceFactory(),
                cacheDataSinkFactory,
                CacheDataSource.FLAG_BLOCK_ON_CACHE,
                null);

        networkMediaSourceFactory = new ExtractorMediaSource.Factory(cacheDataSourceFactory);
    }


    private void buildLocalMediaSourceFactory(Application application){
        FileDataSourceFactory fileDataSourceFactory = new FileDataSourceFactory();
        //创建缓存，指定缓存位置，和缓存策略,为最近最少使用原则,最大为200m
        Cache cache = new SimpleCache(application.getCacheDir(), new LeastRecentlyUsedCacheEvictor(1024 * 1024 * 200));
        //把缓存对象cache和负责缓存数据读取、写入的工厂类CacheDataSinkFactory 相关联
        CacheDataSinkFactory cacheDataSinkFactory = new CacheDataSinkFactory(cache, Long.MAX_VALUE);

        CacheDataSourceFactory cacheDataSourceFactory = new CacheDataSourceFactory(cache,
                fileDataSourceFactory,
                new FileDataSourceFactory(),
                cacheDataSinkFactory,
                CacheDataSource.FLAG_BLOCK_ON_CACHE,
                new CacheDataSource.EventListener() {
                    @Override
                    public void onCachedBytesRead(long cacheSizeBytes, long cachedBytesRead) {
                        Log.e("###视频缓存 ","cachedBytesRead ="+cachedBytesRead + "cacheSizeBytes = "+cacheSizeBytes);
                    }
                });

        localMediaSourceFactory = new ExtractorMediaSource.Factory(cacheDataSourceFactory);
    }



    public static PlayerManager getInstance(){
        if (mInstance == null) {
            synchronized (PlayerManager.class) {
                if (mInstance == null) {
                    mInstance = new PlayerManager();
                }
            }
        }
        return mInstance;
    }

    /**
     * 切换与播放器exoplayer 绑定的exoplayerView。用于页面切换视频无缝续播的场景
     * @param newPlayerView
     * @param attach
     */
    public void switchPlayerView(PlayerView newPlayerView, boolean attach) {
        playerView.setPlayer(attach ? null : exoPlayer);
        newPlayerView.setPlayer(attach ? exoPlayer : null);
    }

    public MediaSource createMediaSource(String url) {
        if(mediaSourceHashMap.containsKey(url)){
            return mediaSourceHashMap.get(url);
        }
        MediaSource mediaSource = null;
        if(url.contains("http")){
            return networkMediaSourceFactory.createMediaSource(Uri.parse(url));
        }else{
            mediaSource = localMediaSourceFactory.createMediaSource(Uri.fromFile(new File(url)));
        }
        mediaSourceHashMap.put(url,mediaSource);
        return mediaSource;
    }
}